/***************************************************************/
/*							                                   */
/*			  POLKA_LOCAL.H			                           */
/*							                                   */
/***************************************************************/
/*    Copyright 1993 Georgia Tech -- John T. Stasko            */


/***************************************************************/
/**********	    include files		      **********/
/***************************************************************/

#ifndef __POLKA_LOCAL_H
#define __POLKA_LOCAL_H

typedef struct {
    int x, y;
} XPoint;


typedef struct {
    int x, y;
    unsigned int width, height;
} XRectangle;

#include <stdlib.h>
#include <iostream.h>
#include <stdio.h>
#include <string.h>
#include <stdarg.h>
#include <limits.h>
#include <math.h>

#include "common_includes.h"
#include "constants.h"
#include "System.h"

typedef  int   LINE_STYLE, FILL_STYLE;
typedef  char *COLOR;
typedef  int   PART;

#define  SOLID	        0
#define  DASHED         1
#define  DOTTED         2
#define  THICK	        3
#define  THICK_DASHED   4
#define  THICK_DOTTED   5
#define  THICKER	6
#define  THICKER_DASHED 7
#define  THICKER_DOTTED 8


#define  PART_C	      0
#define  PART_NW      1
#define  PART_N	      2
#define  PART_NE      3
#define  PART_E	      4
#define  PART_SE      5
#define  PART_S	      6
#define  PART_SW      7
#define  PART_W	      8

/***************************************************************/
/**********	  constant definitions		      **********/
/***************************************************************/

#define   WIDTH             500
#define   HEIGHT            500
#define   MAXSTRINGLENGTH   256
#define   ZERO              0
#define   MYPI              ((double)3.14159)

#define WHITE             0
#define YELLOW            1
#define GREEN             2
#define BLUE              3
#define ORANGE            4
#define RED               5
#define MAROON            6
#define BLACK             7

#define   POLKA_BASE_FILL         0
#define   POLKA_NUM_FILL         41
#define   POLKA_FILL_OUTLINE     POLKA_BASE_FILL
#define   POLKA_FILL_SOLID       POLKA_BASE_FILL + POLKA_NUM_FILL - 1

#define   POLKA_ACTION_POINTS      20
#define   POLKA_ACTION_NUMTYPES    3


#define   MIN(A,B)        ( (A) < (B) ? (A) : (B) )

#ifndef ABS
#define  ABS(A)       ( (A) < 0.0 ? -(A) : (A) )
#endif

#define  BETWEEN(A,X,B) ( ((A) <= (X)) && ((X) <= (B)) )

#define POLKA_complain(msg1,msg2) (printf("Error: %s: %s.\n", (msg1), (msg2)))


typedef  int   COLORINDEX;

typedef void (*TIMER_CALLBACK)(View *, int, void*);


class AONode;
class AnimObject;

typedef struct AONode
{
   class  AnimObject  *object;
   struct AONode      *next;
   struct AONode      *prev;
   struct AONode      *upnext;
   struct AONode      *upprev;
} *AOPtr;

/***************************************************************/
/**********	      local data types		      **********/
/***************************************************************/

typedef struct TimerCBNode
{
   int                 time;
   TIMER_CALLBACK      callbackFct;
   void               *data;
   struct TimerCBNode *next;
} *TimerCBPtr;


typedef struct ImplNode
{
   class AnimObjectImpl *objimpl;
   int                   time;
   int                   existingpos;
   struct ImplNode      *next;
} *ImplPtr;

typedef struct IntNode
{
   int             value;
   struct IntNode *next;
} *IntPtr;


typedef struct ActionDeleteNode
{
   int             value;
   struct Action  *act;
   struct ActionDeleteNode *next;
} *ActionDeletePtr;


typedef struct ActionInfo
{
   int                time;
   struct OFFSET     *offsets;        // always points to start of list
   struct OFFSET     *doing;          // the one currently at
   struct Action     *originalAction; // used for deleting Continuous Actions
   struct Action     *action;         // a copy of the Action being Programmed
   struct ActionInfo *next;
   struct ActionInfo *prev;
 } *ActionInfoPtr;


typedef struct OFFSET        /* used by Actions */
{
   double	  dx;
   double	  dy;
   struct OFFSET *prevo;
   struct OFFSET *nexto;
} *OFFSET_PTR;


typedef struct _COLOR_MAPPER {
   char        name[128];
   COLORINDEX  colornum;
   double      pathval;
   struct _COLOR_MAPPER *next;
   } COLOR_MAPPER;

/***************************************************************/
/**********	    global variables		      **********/
/***************************************************************/

extern   int           _delay;
//extern   Display      *_display;
//extern   XtAppContext  _context;
extern   int           _color_screen;
extern   COLOR_MAPPER *_color_list;
extern   COLOR_MAPPER *_string_list;

void              PolkaInit();
void              BatchMode(int);
void              CheckXEvents();
void              micro_sleep(int);


void SelectPoint(View *xtp, int x, int y);


/******  In polkagc.C  ********/
void              SetBackground(COLORINDEX);
COLORINDEX        load_color(const char * /* COLOR */);
COLORINDEX        set_color(COLORINDEX);
LINE_STYLE        line_style(LINE_STYLE);
FILL_STYLE        fill_style(FILL_STYLE);
int               load_font(const char *);
typedef unsigned long Font;
void              text_info(Font, const char *, int *, int *, int *, int *);
int               set_font(int);
GC                inq_gc();
int               inq_color();
int               inq_font();
void              initGC();


COLORINDEX        get_pathcolor(double, double);
char *            get_pathstring(double, double);
COLORINDEX        get_color(COLOR);
FILL_STYLE        get_fillstyle(double);
LINE_STYLE        get_linestyle(double, double);
void              compute_arrow_off(double, double, double *, double *, 
                                     double *, double *);
void              arrow_poly_direction(int, double[7], double[7], 
                                       double *, double *, double *, double *);

/******* In polkaspline.C *******/
void              make_a_spline(int, int[], int[], View *);

/******** In polkaaux.C ********/
int               streql(const char *, const char *);
int               streqln(const char *, const char *, int);

/******** In polkagroup.C ********/
class AnimObject;
class AnimObjectGroup;

int lineArray(View *, AnimObject *[], int, double, double, double, 
            double, AnimObjectGroup *);
int rectangleArray(View *, AnimObject *[], int, double, double, double, 
            double, AnimObjectGroup *);
int circleArray(View *, AnimObject *[], int, double, double, double, 
            double, AnimObjectGroup *);


#endif
