/***************************************************************/
/*							                                   */
/*	       		polka_staticwindow.h			               */
/*							                                   */
/***************************************************************/
/*       Copyright 1993, 1997 Georgia Institute of Technology 
                     -- John T. Stasko, Roman S. Khramets      */

#ifndef __POLKA_STATIC_WINDOW_H
#define __POLKA_STATIC_WINDOW_H

#include "polka_local.H"

class Shape {
   protected:
      int x,y;
      COLORINDEX color;
   public:
      virtual void draw(/*Window, int, StaticView **/) = 0;
};


class PointShape : public Shape {
   public:
      PointShape(int, int, const char *);
      void draw(/*Window, int, StaticView **/);
};



class LineShape : public Shape {
   private:
      int xs, ys;
   public:
      LineShape(int, int, int, int, const char *);
      void draw(/*Window, int, StaticView **/);
};



class RectangleShape : public Shape {
   private:
      int xs, ys;
   public:
      RectangleShape(int, int, int, int, const char *);
      void draw(/*Window, int, StaticView **/);
};



class EllipseShape : public Shape {
   private:
      int xs, ys;
   public:
      EllipseShape(int, int, int, int, const char *);
      void draw(/*Window, int, StaticView **/);
};


class TextShape : public Shape {
   private:
      int fontid;
      char str[128];
      int xoff,xext,yoff,yext;
   public:
      TextShape(int, int, const char *, const char *, const char *);
      void draw(/*Window, int, StaticView **/);
};



typedef struct ShapeNode
{
   Shape *shape;
   int appeartime;
   struct ShapeNode *prev;
   struct ShapeNode *next;
} *ShapeNodePtr;

#endif
